@@ -92,6 +92,17 @@ int snd_mixer_info( void *handle, snd_mixer_info_t *info )
   return 0;
 }
 
+int snd_mixer_exact_mode( void *handle, int enable )
+{
+  snd_mixer_t *mixer;
+  
+  mixer = (snd_mixer_t *)handle;
+  if ( !mixer ) return -EINVAL;
+  if ( ioctl( mixer -> fd, SND_MIXER_IOCTL_EXACT, &enable ) < 0 )
+    return -errno;
+  return 0;  
+}
+
 int snd_mixer_channel( void *handle, const char *channel_id )
 {
   snd_mixer_t *mixer;
@@ -132,7 +143,7 @@ int snd_mixer_channel_read( void *handle, int channel, snd_mixer_channel_t *data
   mixer = (snd_mixer_t *)handle;
   if ( !mixer ) return -EINVAL;
   data -> channel = channel;
-  if ( ioctl( mixer -> fd, SND_MIXER_IOCTL_CHANNEL_READ, channel ) < 0 )
+  if ( ioctl( mixer -> fd, SND_MIXER_IOCTL_CHANNEL_READ, data ) < 0 )
     return -errno;
   return 0;
 }
@@ -144,7 +155,7 @@ int snd_mixer_channel_write( void *handle, int channel, snd_mixer_channel_t *dat
   mixer = (snd_mixer_t *)handle;
   if ( !mixer ) return -EINVAL;
   data -> channel = channel;
-  if ( ioctl( mixer -> fd, SND_MIXER_IOCTL_CHANNEL_WRITE, channel ) < 0 )
+  if ( ioctl( mixer -> fd, SND_MIXER_IOCTL_CHANNEL_WRITE, data ) < 0 )
     return -errno;
   return 0;
 }
@@ -175,19 +186,23 @@ int snd_mixer_read( void *handle, snd_mixer_callbacks_t *callbacks )
 {
   snd_mixer_t *mixer;
   int idx, result, count;
-  int buffer[ 32 ];
+  unsigned int cmd, tmp;
+  unsigned char buffer[ 64 ];
   
   mixer = (snd_mixer_t *)handle;
   if ( !mixer ) return -EINVAL;
   count = 0;
   while ( (result = read( mixer -> fd, &buffer, sizeof( buffer ))) > 0 ) {
-    if ( result & 3 ) return -EIO;
+    if ( result & 7 ) return -EIO;
     if ( !callbacks ) continue;
-    result >>= 2;
-    for ( idx = 0; idx < result; idx++ )
-      if ( callbacks -> channel_was_changed )
-        callbacks -> channel_was_changed( callbacks -> private_data, buffer[ idx ] );
-    count += result;
+    for ( idx = 0; idx < result; idx += 8 ) {
+      cmd = *(unsigned int *)&buffer[ idx ];
+      tmp = *(unsigned int *)&buffer[ idx + 4 ];
+      if ( cmd == 0 && callbacks -> channel_was_changed ) {        
+        callbacks -> channel_was_changed( callbacks -> private_data, (int)tmp );
+      }
+    }
+    count += result >> 3;	/* return only number of changes */
   }
-  return count;
+  return result >= 0 ? count : -errno;
 }
